import {getThralls, Thrall} from "./thralls.ts";
import {CellView, CheckData} from "./newmap.ts";
import {getPlayerProgress} from "./playerprogress.ts";
import {getHuntMode, HuntMode, initHuntMode} from "./huntmode.ts";
import {generateMap} from "./mapgen.ts";
import {ALL_STATS, Stat} from "./datatypes.ts";
import {D} from "./engine/public.ts";
import {sprLadder, sprLock, sprResourcePickup, sprStatPickup} from "./sprites.ts";
import {GridArt} from "./gridart.ts";
import {getCheckModal} from "./checkmodal.ts";
import {Point} from "./engine/datatypes.ts";

export type Pickup
  = LockPickup
  | StatPickup
  | ExperiencePickup
  | LadderPickup
  | ThrallPickup

export class LockPickup {
  check: CheckData;

  constructor(check: CheckData) {
    this.check = check;
  }

  computeCostToClick() { return 0; }

  isObstructive() { return true; }

  drawFloor() { }
  drawInAir(gridArt: GridArt) {
    for (let z = 0; z < 5; z += 0.25) {
      D.drawSprite(sprLock, gridArt.project(z), 0, {
        xScale: 2.0,
        yScale: 2.0,
      })
    }
  }

  onClick(cell: CellView): boolean {
    getCheckModal().show(this.check, () => cell.pickup = null);
    return true;
  }
}

export class StatPickup {
  stat: Stat;

  constructor(stat: Stat) {
    this.stat = stat;
  }

  computeCostToClick() { return 100; }

  isObstructive() { return true; }

  drawFloor() { }
  drawInAir(gridArt: GridArt) {
    let statIndex = ALL_STATS.indexOf(this.stat);
    if (statIndex == -1) { return; }

    D.drawSprite(
      sprStatPickup,
      gridArt.project(5),
      statIndex,
      {
        xScale: 2,
        yScale: 2,
      }
    )
  }

  onClick(): boolean {
    getPlayerProgress().add(this.stat, 1);
    getPlayerProgress().purloinItem();
    return false;
  }
}

export class ExperiencePickup {
  computeCostToClick() { return 100; }

  isObstructive() { return true; }

  drawFloor() { }
  drawInAir(gridArt: GridArt) {
    D.drawSprite(
      sprResourcePickup,
      gridArt.project(0.0).offset(new Point(0, -16)),
      0,
      {
        xScale: 2,
        yScale: 2,
      }
    );
  }

  onClick(): boolean {
    getPlayerProgress().addExperience(250);
    getPlayerProgress().purloinItem();
    return false;
  }
}

export class LadderPickup {
  computeCostToClick() { return 0; }

  isObstructive() { return false; }

  drawFloor(gridArt: GridArt) {
    D.drawSprite(sprLadder, gridArt.project(0.0), 0, {
      xScale: 2.0,
      yScale: 2.0,
    })
  }
  drawInAir() { }

  onClick(): boolean {
    getPlayerProgress().addBlood(1000);
    initHuntMode(new HuntMode(getHuntMode().depth + 1, generateMap()));
    return false;
  }
}

export class ThrallPickup {
  thrall: Thrall;

  constructor(thrall: Thrall) {
    this.thrall = thrall;
  }

  computeCostToClick() { return 0; }

  isObstructive() { return false; }

  drawFloor() { }
  drawInAir(gridArt: GridArt) {
    let data = getThralls().get(this.thrall);
    D.drawSprite(data.sprite, gridArt.project(0.0), 0, {
      xScale: 2.0,
      yScale: 2.0,
    })
  }

  onClick(cell: CellView): boolean {
    let data = getThralls().get(this.thrall);
    getCheckModal().show(data.initialCheck, () => cell.pickup = null);
    return true;
  }
}